#include "TNR_SHAPE.hxx"
#include "TNR_VECTOR.hxx"
#include "TNR_SURFACE.hxx"
#include "TNR_CURVE.hxx"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

//////////////////////////////
//TNR_BILINEAR_SURFACE
//////////////////////////////


TNR_BILINEAR_SURFACE::TNR_BILINEAR_SURFACE(TNR_POINT  p[4])
{
  for(int i = 0; i < 4; i++){
    _p[i] = p[i];
  }
}

TNR_POINT TNR_BILINEAR_SURFACE::getPosition(double pu,double pv)
{
  double u  = ( pu * 2.0 ) - 1.0;
  double v  = ( pv * 2.0 ) - 1.0;

  double k0 = 0.25*(1-u)*(1-v);
  double k1 = 0.25*(1+u)*(1-v);
  double k2 = 0.25*(1+u)*(1+v);
  double k3 = 0.25*(1-u)*(1+v);
 
  TNR_POINT pos =  k0*_p[0] + k1*_p[1] + k2*_p[2] + k3*_p[3];
                 
  return pos;

}

double TNR_BILINEAR_SURFACE::getUMaximumSideLength()
{
  double value1 = distanceBetween(_p[0],_p[1]);
  double value2 = distanceBetween(_p[2],_p[3]);

  return (value1 < value2) ? value2 : value1;
}

double TNR_BILINEAR_SURFACE::getVMaximumSideLength()
{
  double value1 = distanceBetween(_p[0],_p[3]);
  double value2 = distanceBetween(_p[2],_p[1]);

  return (value1 < value2) ? value2 : value1;
}



//////////////////////////////
//TNR_QUADRIC_SURFACE
//////////////////////////////

TNR_QUADRIC_SURFACE::TNR_QUADRIC_SURFACE(TNR_POINT  p[8])
{
  for(int i = 0; i < 8; i++){
    _p[i] = p[i];
  }
}

TNR_POINT TNR_QUADRIC_SURFACE::getPosition(double u,double v)
{
 
  double k0 = (1-u)*(1-v)*(1-2*u-2*v);
  double k1 = 4*u*(1-u)*(1-v);
  double k2 = u*(1-v)*(2*u-2*v-1);
  double k3 = 4*u*v*(1-v);
  double k4 = u*v*(2*u+2*v-3);
  double k5 = 4*u*(1-u)*v;
  double k6 = (1-u)*v*(2*v-2*u-1);
  double k7 = 4*(1-u)*v*(1-v);

  TNR_POINT pos =   k0*_p[0] + k1*_p[1] + k2*_p[2] + k3*_p[3] + k4*_p[4]
                    + k5*_p[5] + k6*_p[6] + k7*_p[7];
                 
 return pos;

}

double TNR_QUADRIC_SURFACE::getUMaximumSideLength()
{
  TNR_QUADRIC_CURVE curve1(_p[0],_p[1],_p[2]);
  TNR_QUADRIC_CURVE curve2(_p[6],_p[5],_p[4]);
  double value1 = curve1.getLength();
  double value2 = curve2.getLength();
  return (value1 < value2) ? value2 : value1;
}

double TNR_QUADRIC_SURFACE::getVMaximumSideLength()
{
  TNR_QUADRIC_CURVE curve1(_p[2],_p[3],_p[4]);
  TNR_QUADRIC_CURVE curve2(_p[6],_p[7],_p[0]);
  double value1 = curve1.getLength();
  double value2 = curve2.getLength();
  return (value1 < value2) ? value2 : value1;
}



//////////////////////////////
//TNR_CUBIC_SURFACE
//////////////////////////////

TNR_CUBIC_SURFACE::TNR_CUBIC_SURFACE(TNR_POINT  p[8])
{
  for(int i = 0; i < 8; i++){
    _p[i] = p[i];
  }
}

TNR_POINT TNR_CUBIC_SURFACE::getPosition(double u,double v)
{
 double k0 =  0.5*(1.0-u)*(1.0-v)*(2.0-9.0*u*(1.0-u)-9.0*v*(1.0-v));
 double k1 =  4.5*u*(1.0-u)*(2.0-3.0*u)*(1.0-v);
 double k2 =  -4.5*u*(1.0-u)*(1.0-3.0*u)*(1.0-v);
 double k3 =  0.5*u*(1.0-v)*(2.0-9.0*u*(1.0-u)-9.0*v*(1.0-v));

 double k4 =  4.5*u*v*(1.0-v)*(2.0-3.0*v);
 double k5 =  -4.5*u*v*(1.0-v)*(1.0-3.0*v);

 double k6 =  0.5*u*v*(2.0-9*u*(1.0-u)-9*v*(1.0-v));
 double k7 =  -4.5*u*(1.0-u)*(1.0-3.0*u)*v;

 double k8 =  4.5*u*(1.0-u)*(2.0-3.0*u)*v;
 double k9 =  0.5*(1.0-u)*v*(2.0-9*u*(1.0-u)-9.0*v*(1.0-v));
 double k10 = -4.5*(1.0-u)*v*(1.0-v)*(1.0-3.0*v);
 double k11 =  4.5*(1.0-u)*v*(1.0-v)*(2.0-3.0*v);

 TNR_POINT pos =   k0*_p[0] + k1*_p[1] + k2*_p[2]   + k3*_p[3]
				 + k4*_p[4] + k5*_p[5] + k6*_p[6]   + k7*_p[7]
				 + k8*_p[8] + k9*_p[9] + k10*_p[10] + k11*_p[11];
 return pos;

}

double TNR_CUBIC_SURFACE::getUMaximumSideLength()
{
  TNR_CUBIC_CURVE curve1(_p[0],_p[1],_p[2],_p[3]);
  TNR_CUBIC_CURVE curve2(_p[6],_p[7],_p[8],_p[9]);
  double value1 = curve1.getLength();
  double value2 = curve2.getLength();
  return (value1 < value2) ? value2 : value1;
}

double TNR_CUBIC_SURFACE::getVMaximumSideLength()
{
  TNR_CUBIC_CURVE curve1(_p[3],_p[4],_p[5],_p[6]);
  TNR_CUBIC_CURVE curve2(_p[9],_p[10],_p[11],_p[0]);
  double value1 = curve1.getLength();
  double value2 = curve2.getLength();
  return (value1 < value2) ? value2 : value1;
}



//////////////////////////////
//TNR_ELLIPSE_SURFACE
//////////////////////////////

TNR_ELLIPSE_SURFACE::TNR_ELLIPSE_SURFACE(TNR_POINT center,double majorAxisRadius,double minorAxisRadius)
{
  _center = center;
  _a = majorAxisRadius;
  _b = minorAxisRadius;
}

TNR_POINT TNR_ELLIPSE_SURFACE::getPosition(double u,double v)
{
 //u is r,v is rotete angle

 TNR_ELLIPSE e( _center,_a * u, _b * u);
 return e.getPosition(v);

}

double TNR_ELLIPSE_SURFACE::getUMaximumSideLength()
{
  return _a;
}

double TNR_ELLIPSE_SURFACE::getVMaximumSideLength()
{
  TNR_ELLIPSE e( _center,_a,_b);
  return e.getLength();
}



//////////////////////////////
//TNR_CIRCLE_SURFACE
//////////////////////////////

TNR_CIRCLE_SURFACE::TNR_CIRCLE_SURFACE(TNR_POINT center,double radius)
{
  _center = center;
  _r = radius;
}

TNR_POINT TNR_CIRCLE_SURFACE::getPosition(double u,double v)
{
 //u is r,v is rotete angle

 TNR_CIRCLE c(_center,_r * u);
 return c.getPosition(v);

}

double TNR_CIRCLE_SURFACE::getUMaximumSideLength()
{
  return _r;
}

double TNR_CIRCLE_SURFACE::getVMaximumSideLength()
{
  TNR_CIRCLE c( _center,_r);
  return c.getLength();
}



